@@ -1,5 +1,5 @@
 /*
- *  R : A Computer Langage for Statistical Data Analysis
+ *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/*== see ./printutils.c  for general remarks on Printing and the Encode.. utils.
+/*== see ./printutils.c	 for general remarks on Printing and the Encode.. utils.
  *   see ./format.c	 for the  format_Foo_  functions.
  */
 #include "Defn.h"
@@ -25,9 +25,12 @@
 
 SEXP mkChar(char *);
 
-/* do_paste uses two passes to paste the arguments (in CAR(args)) */
-/* together the first pass calculates the width of the paste buffer, */
-/* then it is alloc-ed and the second pass stuffs the information in. */
+/*  .Internal(paste(args, sep, collapse)) */
+
+/* do_paste uses two passes to paste the arguments (in CAR(args)) together.
+ * The first pass calculates the width of the paste buffer,
+ * then it is alloc-ed and the second pass stuffs the information in.
+ */
 SEXP do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 {
 	SEXP ans, collapse, sep, px, x, tmpchar;
@@ -35,7 +38,7 @@ SEXP do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 	char *s, *buf;
 
 	checkArity(op, args);
-	
+
 		/* We use formatting and so we */
 		/* must initialize printing. */
 
@@ -129,12 +132,10 @@ SEXP do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 
 SEXP do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-	SEXP l, nastring, x, y;
-	int i, n, nl, trim;
+	SEXP l, x, y;
+	int i, n, trim;
 	int w, d, e;
-#ifdef COMPLEX_DATA
 	int wi, di, ei;
-#endif
 	char *strp;
 
 	PrintDefaults(env);
@@ -171,39 +172,6 @@ SEXP do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 		}
 		UNPROTECT(1);
 		break;
-	case FACTSXP:
-	case ORDSXP:
-		PROTECT(y = allocVector(STRSXP, n));
-		l = getAttrib(x, R_LevelsSymbol);
-		nl = LEVELS(x);
-		if(l == R_NilValue) {
-			PROTECT(l = allocVector(STRSXP, nl));
-			for(i=0 ; i<nl ; i++) {
-				strp = Rsprintf("%d", i+1);
-				STRING(l)[i] = mkChar(strp);
-			}
-		}
-		else {
-			l = duplicate(l);
-			PROTECT(l);
-		}
-		nastring = NA_STRING;
-		if(!trim) {
-			formatString(STRING(l), nl, &w, 0);
-			for(i=0 ; i<nl ; i++) {
-				strp = EncodeString(CHAR(STRING(l)[i]), w, 0);
-				STRING(l)[i] = mkChar(strp);
-			}
-			nastring = mkChar(EncodeString(CHAR(nastring), w, 0));
-		}
-		for(i=0 ; i<n ; i++) {
-			if(INTEGER(x)[i] < 1 || INTEGER(x)[i] > nl)
-				STRING(y)[i] = nastring;
-			else
-				STRING(y)[i] = STRING(l)[INTEGER(x)[i]-1];
-		}
-		UNPROTECT(2);
-		break;
 	case INTSXP:
 		PROTECT(y = allocVector(STRSXP, n));
 		if (trim) w = 0;
@@ -224,7 +192,6 @@ SEXP do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 		}
 		UNPROTECT(1);
 		break;
-#ifdef COMPLEX_DATA
 	case CPLXSXP:
 		formatComplex(COMPLEX(x), n, &w, &d, &e, &wi, &di, &ei);
 		if (trim) wi = w = 0;
@@ -235,13 +202,12 @@ SEXP do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 		}
 		UNPROTECT(1);
 		break;
-#endif
 	case STRSXP:
 		formatString(STRING(x), n, &w, 0);
 		if (trim) w = 0;
 		PROTECT(y = allocVector(STRSXP, n));
 		for (i = 0; i < n; i++) {
-			strp = EncodeString(CHAR(STRING(x)[i]), w, 0);
+			strp = EncodeString(CHAR(STRING(x)[i]), w, 0, adj_left);
 			STRING(y)[i] = mkChar(strp);
 		}
 		UNPROTECT(1);
@@ -259,16 +225,14 @@ SEXP do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 /* format.info(obj)  --> 3 integers  (w,d,e) with the formatting information
  *			w = total width (#{chars}) per item
  *			d = #{digits} to RIGHT of "."
- *			e = {0:2}.   0: Fixpoint;  
+ *			e = {0:2}.   0: Fixpoint;
  *				   1,2: exponential with 2/3 digit expon.
  */
 SEXP do_formatinfo(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-	SEXP l, x;
+	SEXP x;
 	int n, w, d, e;
-#ifdef COMPLEX_DATA
 	int wi, di, ei;
-#endif
 	checkArity(op, args);
 	x = CAR(args);
 	n = LENGTH(x);
@@ -280,24 +244,17 @@ SEXP do_formatinfo(SEXP call, SEXP op, SEXP args, SEXP env)
 		case LGLSXP:
 			formatLogical(LOGICAL(x), n, &w);
 			break;
-		case FACTSXP:
-		case ORDSXP:
-			l = getAttrib(x, R_LevelsSymbol);
-			formatFactor(INTEGER(x), n, &w, l, LEVELS(x));
-			break;
 		case INTSXP:
 			formatInteger(INTEGER(x), n, &w);
 			break;
 		case REALSXP:
 			formatReal(REAL(x), n, &w, &d, &e);
 			break;
-#ifdef COMPLEX_DATA
 		case CPLXSXP:
 			wi = di = ei = 0;
 			formatComplex(COMPLEX(x), n, &w, &d, &e, &wi, &di, &ei);
 			n = -1;/* complex 'code' */
 			break;
-#endif
 		case STRSXP:
 			formatString(STRING(x), n, &w, 0);
 			break;
