@@ -33,7 +33,7 @@ SEXP do_split(SEXP call, SEXP op, SEXP args, SEXP env)
 
 	if(!isFactor(f))
 		errorcall(call, "second argument must be a factor\n");
-	nlevs = LEVELS(f);
+	nlevs = nlevels(f);
 
         nfac=LENGTH(CADR(args));
 	nobs=LENGTH(CAR(args));
@@ -42,10 +42,10 @@ SEXP do_split(SEXP call, SEXP op, SEXP args, SEXP env)
 		return R_NilValue;
 
 	if( nfac<=0 )
-		errorcall(call,"Group length is 0 but data length > 0");
+		errorcall(call, "Group length is 0 but data length > 0");
 	
 	if( nobs != nfac  )
-		warningcall(call,"argument lengths differ\n");
+		warningcall(call, "argument lengths differ\n");
 
 	PROTECT(counts = allocVector(INTSXP, nlevs));
 	for(i=0 ; i<nlevs ; i++)
@@ -56,12 +56,16 @@ SEXP do_split(SEXP call, SEXP op, SEXP args, SEXP env)
 			INTEGER(counts)[j-1] += 1;
 		}
 	}
-	/* allocate pointers */
-	PROTECT(vec = allocVector(STRSXP, nlevs));
+
+		/* Allocate a generic vector to hold the results. */
+		/* The i-th element will hold the split-out data */
+		/* for the ith group. */
+
+	PROTECT(vec = allocVector(VECSXP, nlevs));
 	for(i=0 ; i<nlevs ; i++) {
-		STRING(vec)[i] = allocVector(TYPEOF(x), INTEGER(counts)[i]);
-		LEVELS(STRING(vec)[i])=LEVELS(x);
-		setAttrib(STRING(vec)[i],R_LevelsSymbol,getAttrib(x,R_LevelsSymbol));
+		VECTOR(vec)[i] = allocVector(TYPEOF(x), INTEGER(counts)[i]);
+		setAttrib(VECTOR(vec)[i], R_LevelsSymbol,
+			getAttrib(x, R_LevelsSymbol));
 	}
 	for(i=0 ; i<nlevs ; i++)
 		INTEGER(counts)[i] = 0;
@@ -71,33 +75,38 @@ SEXP do_split(SEXP call, SEXP op, SEXP args, SEXP env)
 			k = INTEGER(counts)[j-1];
 			switch(TYPEOF(x)) {
 				case LGLSXP:
-				case FACTSXP:
-				case ORDSXP:
 				case INTSXP:
-					INTEGER(STRING(vec)[j-1])[k] = INTEGER(x)[i];
+					INTEGER(VECTOR(vec)[j-1])[k] = INTEGER(x)[i];
 					break;
 				case REALSXP:
-					REAL(STRING(vec)[j-1])[k] = REAL(x)[i];
+					REAL(VECTOR(vec)[j-1])[k] = REAL(x)[i];
 					break;
 				case CPLXSXP:
-					COMPLEX(STRING(vec)[j-1])[k] = COMPLEX(x)[i];
+					COMPLEX(VECTOR(vec)[j-1])[k] = COMPLEX(x)[i];
 					break;
 				case STRSXP:
-					STRING(STRING(vec)[j-1])[k] = STRING(x)[i];
+					STRING(VECTOR(vec)[j-1])[k] = STRING(x)[i];
 					break;
 			}
 			INTEGER(counts)[j-1] += 1;
 		}
 	}
+
+		/* Now transfer the results from the vector */
+		/* into a dotted-pair list.  When structures */
+		/* are full based on vectors this won't be needed. */
+
 	PROTECT(ans = allocList(nlevs));
 	x = ans;
 	for(i=0 ; i<nlevs ; i++) {
-		CAR(x) = STRING(vec)[i];
+		CAR(x) = VECTOR(vec)[i];
 		x = CDR(x);
 	}
 	UNPROTECT(3);
 	x = ans;
 	if((vec = getAttrib(f, R_LevelsSymbol)) != R_NilValue) {
+		if (!isString(vec))
+			errorcall(call, "non character factor levels!\n");
 		for(i=0 ; i<nlevs ; i++) {
 			TAG(x) = install(CHAR(STRING(vec)[i]));
 			x = CDR(x);
