@@ -19,7 +19,8 @@
 
 #include <stdio.h>
 #include <ctype.h>
-#include "PostScript.h"
+#include "PS.h"
+#include "Fileio.h"
 
 	/*  PostScript Graphics Utilities			  */
 	/*  This file contains all those utilities used to draw   */
@@ -160,7 +161,7 @@ static int GetCharInfo(char *buf, FontMetricInfo *metrics)
 	if(!MatchKey(buf, "C ")) return 0;
 	p = SkipToNextItem(p);
 	sscanf(p, "%d", &nchar);
-	if(nchar < 0) return 0;
+	if(nchar < 0) return 1;
 	p = SkipToNextKey(p);
 
 	if(!MatchKey(p, "WX")) return 0;
@@ -199,7 +200,7 @@ int PostScriptLoadFontMetrics(char *fontname, FontMetricInfo *metrics)
 	int mode;
 	FILE *fp;
 
-	if(!(fp = fopen(fontname, "r"))) return 0;
+	if(!(fp = R_fopen(fontname, "r"))) return 0;
 
 	mode = 0;
 	while(fgets(buf, BUFSIZE, fp)) {
@@ -238,7 +239,7 @@ error:
 	return 0;
 }
 
-double PostScriptStringWidth(char *p, FontMetricInfo *metrics)
+double PostScriptStringWidth(unsigned char *p, FontMetricInfo *metrics)
 {
 	int sum = 0;
 	for( ; *p ; p++) {
@@ -250,6 +251,22 @@ double PostScriptStringWidth(char *p, FontMetricInfo *metrics)
 	return 0.001 * sum;
 }
 
+void PostScriptMetricInfo(int c, double *ascent, double *descent,
+	double *width, FontMetricInfo *metrics)
+{
+	if(c == 0) {
+		*ascent = 0.001 * metrics->FontBBox[3];
+		*descent = -0.001 * metrics->FontBBox[1];
+		*width = 0.001 * (metrics->FontBBox[2] - metrics->FontBBox[0]);
+
+	}
+	else {
+		*ascent = 0.001 * metrics->CharInfo[c].BBox[3];
+		*descent = -0.001 * metrics->CharInfo[c].BBox[1];
+		*width = 0.001 * metrics->CharInfo[c].WX;
+	}
+}
+
 
 		/*  Part 2.  Graphics Support Code.  */
 
@@ -274,8 +291,8 @@ void PostScriptFileHeader(
 	double top)
 {
 	fprintf(fp, "%%!PS-Adobe-3.0\n");
-	fprintf(fp, "%%%%DocumentFonts: %s %s %s\n", font[0], font[1], font[2]);
-	fprintf(fp, "%%%%+ %s %s\n", font[3], font[4]);
+	fprintf(fp, "%%%%DocumentFonts: %s %s %s\n", font[2], font[4], font[6]);
+	fprintf(fp, "%%%%+ %s %s\n", font[8], font[10]);
 	fprintf(fp, "%%%%DocumentMedia: %s %.0f %.0f 0 ()\n",
 		papername, paperwidth, paperheight);
 	fprintf(fp, "%%%%Title: R Graphics Output\n");
@@ -322,11 +339,12 @@ void PostScriptFileHeader(
 	fprintf(fp, "       closepath clip newpath } def\n");
 	fprintf(fp, "/rgb { setrgbcolor } def\n");
 	fprintf(fp, "/s   { scalefont setfont } def\n");
-	fprintf(fp, "/R   { /%s findfont } def\n", font[0]);
-	fprintf(fp, "/B   { /%s findfont } def\n", font[1]);
-	fprintf(fp, "/I   { /%s findfont } def\n", font[2]);
-	fprintf(fp, "/BI  { /%s findfont } def\n", font[3]);
-	fprintf(fp, "/S   { /%s findfont } def\n", font[4]);
+	fprintf(fp, "/R   { /%s findfont } def\n", font[2]);
+	fprintf(fp, "/B   { /%s findfont } def\n", font[4]);
+	fprintf(fp, "/I   { /%s findfont } def\n", font[6]);
+	fprintf(fp, "/BI  { /%s findfont } def\n", font[8]);
+	fprintf(fp, "/S   { /%s findfont } def\n", font[10]);
+	fprintf(fp, "1 setlinecap 1 setlinejoin\n");
 	fprintf(fp, "%%%%EndProlog\n");
 }
 
@@ -370,12 +388,12 @@ void PostScriptSetColor(FILE *fp, double r, double g, double b)
 	fprintf(fp,"%.4f %.4f %.4f rgb\n", r, g, b);
 }
 
-void PostScriptSetLineTexture(FILE *fp, int *lty, int nlty)
+void PostScriptSetLineTexture(FILE *fp, int *lty, int nlty, double lwd)
 {
 	int i;
 	fprintf(fp,"[");
 	for(i=0 ; i<nlty ; i++)
-		fprintf(fp," %d", lty[i]);
+		fprintf(fp," %.2f", lty[i] * lwd);
 	fprintf(fp,"] 0 setdash\n");
 }
 
@@ -421,6 +439,7 @@ void PostScriptOpenCircle(FILE *fp, double x, double y, double r)
 	fprintf(fp, "%.2f %.2f %.2f co\n", x, y, r);
 }
 
+#ifdef GONE
 void PostScriptFilledPolygon(FILE *fp, double *x, double *y, int nxy)
 {
 	int i;
@@ -430,16 +449,31 @@ void PostScriptFilledPolygon(FILE *fp, double *x, double *y, int nxy)
 		fprintf(fp, "  %.2f %.2f l\n", x[i], y[i]);
 	fprintf(fp, "cp f\n");
 }
+#endif
 
+#ifdef GONE
 void PostScriptOpenPolygon(FILE *fp, double *x, double *y, int nxy)
 {
 	int i;
 	fprintf(fp, "np\n");
-	fprintf(fp, "  %.2f %.2f m\n", x[0], y[0]);
+	fprintf(fp, "%.2f %.2f m\n", x[0], y[0]);
 	for(i=1 ; i<nxy ; i++)
-		fprintf(fp, "  %.2f %.2f l\n", x[i], y[i]);
+		fprintf(fp, "%.2f %.2f l\n", x[i], y[i]);
 	fprintf(fp, "cp o\n");
 }
+#endif
+
+#ifdef GONE
+void PostScriptPolyline(FILE *fp, double *x, double *y, int nxy)
+{
+	int i;
+	fprintf(fp, "np\n");
+	fprintf(fp, "%.2f %.2f m\n", x[0], y[0]);
+	for(i=1 ; i<nxy ; i++)
+		fprintf(fp, "%.2f %.2f l\n", x[i], y[i]);
+	fprintf(fp, "o\n");
+}
+#endif
 
 static void PostScriptWriteString(FILE *fp, char *str)
 {
