@@ -124,7 +124,7 @@ LoadedDLL[MAX_NUM_DLLS];
 	/* Returns 1 if the DLL was found and removed from */
 	/* the list and returns 0 otherwise. */
 
-static DeleteDLL(char *path)
+static int DeleteDLL(char *path)
 {
 	int i, loc;
 	for(i=0 ; i<CountDLL ; i++) {
@@ -151,7 +151,7 @@ found:
 	/* and returns 0 if there library table is full or */
 	/* or if dlopen fails for some reason. */
 
-static AddDLL(char *path)
+static int AddDLL(char *path)
 {
 	void *handle;
 	char *dpath;
@@ -216,7 +216,7 @@ DL_FUNC R_FindSymbol(char const *name)
 }
 
 
-static GetFullDLLPath(SEXP call, char *buf, char *path)
+static void GetFullDLLPath(SEXP call, char *buf, char *path)
 {
 	if(path[0] != '/') {
 		if(!getcwd(buf, MAXPATHLEN))
